package betterwithmods.common.blocks.tile;

import betterwithmods.util.InvUtils;
import betterwithmods.util.WorldUtils;
import net.minecraft.block.state.IBlockState;
import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.init.Items;
import net.minecraft.item.ItemStack;
import net.minecraft.nbt.NBTTagCompound;
import net.minecraft.tileentity.TileEntity;
import net.minecraft.util.EnumFacing;
import net.minecraft.util.EnumHand;
import net.minecraft.util.math.BlockPos;
import net.minecraft.world.World;

import javax.annotation.Nonnull;
import java.util.Optional;

/**
 * Created by tyler on 5/17/17.
 */
public class TileStake extends TileBasicInventory {

    public static Optional<TileStake> getStake(@Nonnull World world, @Nonnull BlockPos pos) {
        TileEntity tile = world.func_175625_s(pos);
        return Optional.ofNullable(tile instanceof TileStake ? (TileStake) tile : null);
    }

    public BlockPos[] connections = new BlockPos[getInventorySize()];

    public boolean onBlockActivated(World worldIn, BlockPos pos, IBlockState state, EntityPlayer playerIn, EnumHand hand, EnumFacing side, float hitX, float hitY, float hitZ) {
        connectStake(worldIn, pos, side.func_176734_d(), playerIn);
        return true;
    }

    public int getInventorySize() {
        return 6;
    }

    public BlockPos findStake(World world, BlockPos first, EnumFacing direction) {
        BlockPos second = first;
        int i = 0;
        while (i < 64) {
            second = second.func_177972_a(direction);
            if (getStake(world, second).isPresent()) {
                return second;
            }
            i++;
        }
        return null;
    }

    public boolean isConnected(EnumFacing facing) {
        return connections[facing.func_176745_a()] != null;
    }

    public ItemStack getItemFromDistance(BlockPos pos1, BlockPos pos2) {
        return new ItemStack(Items.field_151007_F, WorldUtils.getDistance(pos1, pos2));
    }

    public void setConnected(EnumFacing facing, BlockPos pos) {
        this.connections[facing.func_176745_a()] = pos;
        if (pos == null) {
            inventory.setStackInSlot(facing.func_176745_a(), ItemStack.field_190927_a);
        } else {
            inventory.setStackInSlot(facing.func_176745_a(), getItemFromDistance(func_174877_v(), pos));
        }
        func_70296_d();
    }

    public boolean connectStake(World world, BlockPos first, EnumFacing direction, EntityPlayer player) {
        BlockPos second = findStake(world, first, direction);
        if (first == null || second == null)
            return false;
        ItemStack string = getItemFromDistance(first, second);

        TileStake firstTile = getStake(world, first).orElse(null), secondTile = getStake(world, second).orElse(null);
        if (firstTile == null || secondTile == null)
            return false;
        if (!firstTile.isConnected(direction) && !secondTile.isConnected(direction)) {
            if (!InvUtils.usePlayerItemStrict(player, EnumFacing.UP, string, string.func_190916_E()))
                return false;
            firstTile.setConnected(direction, second);
            secondTile.setConnected(direction.func_176734_d(), first);
        }
        return false;
    }

    @Override
    public void onBreak() {
        for (int i = 0; i < connections.length; i++) {
            BlockPos second = connections[i];
            if (second != null) {
                TileStake stake = getStake(field_145850_b, second).orElse(null);
                if (stake != null) {
                    stake.setConnected(EnumFacing.func_82600_a(i).func_176734_d(), null);
                }
            }
        }
        super.onBreak();
    }

    @Override
    public NBTTagCompound func_189515_b(NBTTagCompound tag) {
        for (EnumFacing facing : EnumFacing.field_82609_l) {
            if (connections[facing.func_176745_a()] != null)
                tag.func_74772_a(getKey(facing), connections[facing.func_176745_a()].func_177986_g());
        }
        return super.func_189515_b(tag);
    }

    @Override
    public void func_145839_a(NBTTagCompound compound) {
        for (EnumFacing facing : EnumFacing.field_82609_l) {
            if (compound.func_74764_b(getKey(facing)))
                connections[facing.func_176745_a()] = BlockPos.func_177969_a(compound.func_74763_f(getKey(facing)));
        }
        super.func_145839_a(compound);
    }

    public String getKey(EnumFacing facing) {
        return "connected" + facing.name();
    }
}
